import { world, system, EquipmentSlot, ItemStack, Block, EntityEquippableComponent, ItemEnchantableComponent } from "@minecraft/server";
import { Durability } from "./ToolDurability.js";
import { LavaResistance, NETHERITE_HAMMER_ID } from "./LavaResistance.js";

export const HAMMER_IDENTIFIER = [
    "hammertool:wooden_hammer",
    "hammertool:stone_hammer",
    "hammertool:iron_hammer",
    "hammertool:golden_hammer",
    "hammertool:diamond_hammer",
    NETHERITE_HAMMER_ID,
];

const hammerDestroyListTags = [
    "stone",
    "metal"
];
const hammerDestroyListIDs = [
    "minecraft:deepslate",
    "obsidian",
    "minecraft:calcite"
];

world.beforeEvents.playerBreakBlock.subscribe(async (data) => {
    const player = data.player;
    const block = data.block;

    const equippableComponent = player.getComponent(EntityEquippableComponent.componentId);
    const item = equippableComponent.getEquipment(EquipmentSlot.Mainhand);

    if (item && HAMMER_IDENTIFIER.includes(item.typeId)) {
        let blocksDestroyed = player.isSneaking ? await useHammer(player, block, item) : 1;
        const maxDurability = item.getComponent('durability').maxDurability;
        blocksDestroyed = Math.min(blocksDestroyed, maxDurability);

        system.run(() => {
            Durability(player, item, blocksDestroyed);
        });
    }
});

async function useHammer(player, block, item) {
    if (!player || !block) {
        return 0;
    }

    const viewDirection = player.getViewDirection();
    const mainDirection = getMainDirection(viewDirection);
    const positions = getPositionsToDestroy(block.location, mainDirection);

    let blocksDestroyed = 1;
    const enchantments = item.getComponent(ItemEnchantableComponent.componentId);
    const hasSilkTouch = enchantments.hasEnchantment('silk_touch');

    for (const pos of positions) {
        const targetBlock = player.dimension.getBlock(pos);
        if (!targetBlock || (pos.x === block.x && pos.y === block.y && pos.z === block.z)) continue;

        if (canDestroyBlock(targetBlock)) {
            const x = Math.floor(pos.x);
            const y = Math.floor(pos.y);
            const z = Math.floor(pos.z);

            if (hasSilkTouch) {
                const blockTypeId = targetBlock.typeId;
                if (blockTypeId) {
                    await player.runCommandAsync(`setblock ${x} ${y} ${z} air replace`);
                    player.dimension.spawnItem(new ItemStack(blockTypeId, 1), { x, y, z });
                    player.playSound("dig." + blockTypeId.split(":")[1], {
                        location: { x: x, y: y, z: z },
                        volume: 1.0,
                        pitch: 1.0
                    });
                }
            } else {
                await player.runCommandAsync(`setblock ${x} ${y} ${z} air destroy`);
            }

            blocksDestroyed++;
        }
    }

    return blocksDestroyed;
}

function canDestroyBlock(block) {
    for (const tag of hammerDestroyListTags) {
        if (block.hasTag(tag)) return true;
    }

    if (hammerDestroyListIDs.includes(block.type.id)) return true;

    const itemStack = block.getItemStack();
    if (itemStack && itemStack.typeId.match(/brick|_wall|tuff|basalt|blackstone|sandstone|_stone|_deepslate|obsidian|quartz|mud|purpur|prismarine/)) return true;

    return false;
}

function getMainDirection(viewDirection) {
    const absX = Math.abs(viewDirection.x);
    const absY = Math.abs(viewDirection.y);
    const absZ = Math.abs(viewDirection.z);

    if (absX > absY && absX > absZ) return viewDirection.x > 0 ? "east" : "west";
    if (absZ > absX && absZ > absY) return viewDirection.z > 0 ? "south" : "north";
    return viewDirection.y > 0 ? "up" : "down";
}

function getPositionsToDestroy(center, mainDirection) {
    const positions = [];
    const x = Math.floor(center.x);
    const y = Math.floor(center.y);
    const z = Math.floor(center.z);

    switch (mainDirection) {
        case "north":
        case "south":
            for (let dx = -1; dx <= 1; dx++) {
                for (let dy = -1; dy <= 1; dy++) {
                    positions.push({ x: x + dx, y: y + dy, z: z });
                }
            }
            break;
        case "east":
        case "west":
            for (let dz = -1; dz <= 1; dz++) {
                for (let dy = -1; dy <= 1; dy++) {
                    positions.push({ x: x, y: y + dy, z: z + dz });
                }
            }
            break;
        case "up":
        case "down":
            for (let dx = -1; dx <= 1; dx++) {
                for (let dz = -1; dz <= 1; dz++) {
                    positions.push({ x: x + dx, y: y, z: z + dz });
                }
            }
            break;
    }

    return positions;
}

system.run(() => {
    console.warn("Hammer script initialized");
    LavaResistance()
});